//---------------------------------------------------------------------------------------
/// \file	RectangleSelectTool.cpp
/// \author	Rob Bateman
/// \brief	Impliments a very simple rectangular selection tool. For tools, we require
/// 		both MPxContext and MPxContextCommand nodes. To understand this idea, think
/// 		that each tool in maya has to be able to handle user input that occurs within
/// 		the viewport.
///
/// 			For example, when we have the select tool in use, mouse clicks within Maya's
/// 		viewport will be passed to the select tool to interpret. This means that when
/// 		we click in a view, the action performed is to select an object. Say we then
/// 		switch to the splitPolygonTool, mouse clicks in the view now create small dots
/// 		on mesh edges until we complete the tool.
///
/// 			The MPxContext then, forms the base class of the user input part of a custom
/// 		tool. ie, the context class we provide handles all of the mouse presses and drags
/// 		etc.
///
///				MPxContextCommand is a far simpler beast. This is simply a mel command that
/// 		when executed by maya, will tell it to use our custom tool (context). The makeObj
/// 		function therefore just needs to be overloaded to return a new instance of our
/// 		custom context.
///
//---------------------------------------------------------------------------------------

#include "RectangleSelectTool.h"

//---------------------------------------------------------------------------------------	RectangleSelectToolCommand :: makeObj()
/// \brief	This just needs to return a new instance of our tools context. This is used
/// 		by maya to set our tool as current.
/// \return	a new instance of the tool
///
MPxContext* RectangleSelectToolCommand::makeObj()
{
	return new RectangleSelectTool;
}

//---------------------------------------------------------------------------------------	RectangleSelectToolCommand :: creator()
/// \brief	This is used by maya to create a new instance of the tool command
/// \return	a new RectangleSelectContextCmd instance
///
void* RectangleSelectToolCommand::creator()
{
	return new RectangleSelectToolCommand;
}

/// some help text to display in the help line UI element
const char* g_HelpString = "Click and Drag rectangular region";

//---------------------------------------------------------------------------------------	RectangleSelectTool :: RectangleSelectTool()
/// \brief	ctor
///
RectangleSelectTool::RectangleSelectTool()
{
	setTitleString ( "Rectangle Selection Tool" );
	m_IsSelecting=false;
}

//---------------------------------------------------------------------------------------	RectangleSelectTool :: ~RectangleSelectTool()
/// \brief	dtor
///
RectangleSelectTool::~RectangleSelectTool()
{
}

//---------------------------------------------------------------------------------------	RectangleSelectTool :: toolOnSetup()
/// \brief	This is used to setup any user interface things you need when the tool
/// 		is made current.
/// \param	event	-	information on the input event
///
void RectangleSelectTool::toolOnSetup ( MEvent & )
{
	setHelpString( g_HelpString );	//	sets the help text in the help UI item
	setCursor( MCursor::pencilCursor );
}

//---------------------------------------------------------------------------------------	RectangleSelectTool :: doPress()
/// \brief	overide to handle a mouse press event
/// \param	event	-	information on the input event
/// \return	MS::kSuccess or MS::kFailure
///
MStatus RectangleSelectTool::doPress( MEvent & event )
{
	// if we have a left mouse click, start the selection of
	// our rectangular region
	//
	if(event.mouseButton() == MEvent::kLeftMouse)
	{

		// We need to keep a reference of the view in which this command
		// was called. This basically lets us know which viewport to draw
		// in...
		m_View = M3dView::active3dView();

		// we'll only handle the event if the left mouse button is the only thing
		// held down.
		if( !event.isModifierShift() &&
			!event.isModifierControl() &&
			!event.isModifierMiddleMouseButton() )
		{
			// get the current position of the cursor. We need to know this so we can
			// start drawing our selection rectangle!
			event.getPosition( m_Start[0], m_Start[1] );

			// make sure both the start and end points are in the same place.
			m_End[0] = m_Start[0];
			m_End[1] = m_Start[1];


			m_IsSelecting = true;

			setCursor( MCursor::doubleCrossHairCursor );

			// yay!
			return MS::kSuccess;
		}
	}

	// just let the base class handle the event
	return MPxContext::doPress(event);
}

//---------------------------------------------------------------------------------------	RectangleSelectTool :: doDrag()
/// \brief	overide to handle a mouse drag event
/// \param	event	-	information on the input event
/// \return	MS::kSuccess or MS::kFailure
///
MStatus RectangleSelectTool::doDrag( MEvent & event )
{
	// if we are dragging and left mouse button is pressed...
	if(m_IsSelecting)
	{
		// get the new location of the cursor
		event.getPosition( m_End[0], m_End[1] );

		// to draw the selection rectangle within maya, we need to
		// draw a custom overlay in the viewport in which the tool
		// is being used.
		//
		// This means we have to surround the
		m_View.beginOverlayDrawing();

			// clear the overlay planes
			m_View.clearOverlayPlane();

			// Set up the orthographic projection matix
			glMatrixMode( GL_PROJECTION );
			glLoadIdentity();
			gluOrtho2D( 0.0f, m_View.portWidth(),
						0.0f, m_View.portHeight() );

			glMatrixMode( GL_MODELVIEW );
			glLoadIdentity();

			// Set line width
			glLineWidth( 1.0 );

			// Draw  rectangle
			glBegin( GL_LINE_LOOP );
				glVertex2i( m_Start[0], m_Start[1] );
				glVertex2i( m_End[0], m_Start[1] );
				glVertex2i( m_End[0], m_End[1] );
				glVertex2i( m_Start[0], m_End[1] );
			glEnd();

		// done drawing our overlay on the viewport
		m_View.endOverlayDrawing();
	}

	return MPxContext::doDrag(event);
}

//---------------------------------------------------------------------------------------	RectangleSelectTool :: doRelease()
/// \brief	overide to handle a mouse release event
/// \param	event	-	information on the input event
/// \return	MS::kSuccess or MS::kFailure
///
MStatus RectangleSelectTool::doRelease( MEvent & event )
{
	// only bother handling the release of a left mouse button.
	if(event.mouseButton() != MEvent::kLeftMouse) {
		return MPxContext::doRelease(event);
	}

	setCursor( MCursor::pencilCursor );

	// indicate we have finished selecting stuff
	m_IsSelecting = false;

	// Clear the overlay plane & restore from overlay drawing
	m_View.beginOverlayDrawing();
		m_View.clearOverlayPlane();
	m_View.endOverlayDrawing();

	// since the selection area can be dragged in -ve x and -y axes,
	// we need to allow for this and calculate a min and max point
	// (for that is what the selection mechanism will need)
	//
	short Min[2];
	short Max[2];

	if(m_End[0] < m_Start[0]) {
		Min[0] = m_End[0];
		Max[0] = m_Start[0];
	}
	else {
		Max[0] = m_End[0];
		Min[0] = m_Start[0];
	}

	if(m_End[1] < m_Start[1]) {
		Min[1] = m_End[1];
		Max[1] = m_Start[1];
	}
	else {
		Max[1] = m_End[1];
		Min[1] = m_Start[1];
	}

	// Use the selection mechanism within MGlobal to select the items within the
	//
	MGlobal::selectFromScreen( Min[0], Min[1], Max[0], Max[1], MGlobal::kReplaceList );

	// nullify selection area
	m_Start[0] = m_Start[1] = m_End[0] = m_End[1] = 0;

	//
	return MS::kSuccess;
}

