//---------------------------------------------------------------------------------------
/// \file	RectangleSelectTool.h
/// \author	Rob Bateman
/// \brief	Impliments a very simple rectangular selection tool. For tools, we require
/// 		both MPxContext and MPxContextCommand nodes. To understand this idea, think
/// 		that each tool in maya has to be able to handle user input that occurs within
/// 		the viewport.
///
/// 			For example, when we have the select tool in use, mouse clicks within Maya's
/// 		viewport will be passed to the select tool to interpret. This means that when
/// 		we click in a view, the action performed is to select an object. Say we then
/// 		switch to the splitPolygonTool, mouse clicks in the view now create small dots
/// 		on mesh edges until we complete the tool.
///
/// 			The MPxContext then, forms the base class of the user input part of a custom
/// 		tool. ie, the context class we provide handles all of the mouse presses and drags
/// 		etc.
///
///				MPxContextCommand is a far simpler beast. This is simply a mel command that
/// 		when executed by maya, will tell it to use our custom tool (context). The makeObj
/// 		function therefore just needs to be overloaded to return a new instance of our
/// 		custom context.
///
//---------------------------------------------------------------------------------------

#ifndef __RECTANGLE_SELECT_TOOL__H__
#define __RECTANGLE_SELECT_TOOL__H__

	#ifdef WIN32
		#define NT_PLUGIN
	#endif

	#include <maya/MIOStream.h>
	#include <math.h>
	#include <stdlib.h>

	#include <maya/MString.h>
	#include <maya/MGlobal.h>
	#include <maya/M3dView.h>
	#include <maya/MDagPath.h>

	#include <maya/MItSelectionList.h>

	#include <maya/MPxContextCommand.h>
	#include <maya/MPxContext.h>
	#include <maya/MCursor.h>
	#include <maya/MEvent.h>

	#include <GL/gl.h>
	#include <GL/glu.h>

	#include <maya/MPoint.h>
	#include <maya/MPointArray.h>
	#include <maya/MItCurveCV.h>
	#include <maya/MItSurfaceCV.h>
	#include <maya/MItMeshVertex.h>
	#include <maya/MItMeshEdge.h>
	#include <maya/MItMeshPolygon.h>


	//-----------------------------------------------------------------------------------	RectangleSelectTool
	///	\brief	The main part of a tool plugin is the Context part. This is
	/// 		the class that handles the mouse interaction from the user
	/// 		for your tool. In this fairly simple example, the class simply
	/// 		stores and updates a start and end point between which a
	/// 		rectangle will be drawn.
	///
	class RectangleSelectTool
		: public MPxContext
	{
	public:

		/// \brief	ctor
		RectangleSelectTool();

		/// \brief	dtor
		virtual ~RectangleSelectTool();

		/// \brief	This is used to setup any user interface things you need when the tool
		/// 		is made current.
		/// \param	event	-	information on the input event
		///
		virtual void	toolOnSetup( MEvent & event );

		/// \brief	overide to handle a mouse press event
		/// \param	event	-	information on the input event
		/// \return	MS::kSuccess or MS::kFailure
		///
		virtual MStatus	doPress( MEvent & event );

		/// \brief	overide to handle a mouse drag event
		/// \param	event	-	information on the input event
		/// \return	MS::kSuccess or MS::kFailure
		///
		virtual MStatus	doDrag( MEvent & event );

		/// \brief	overide to handle a mouse release event
		/// \param	event	-	information on the input event
		/// \return	MS::kSuccess or MS::kFailure
		///
		virtual MStatus	doRelease( MEvent & event );
		
	private:

		/// is the selection tool selecting things?
		bool m_IsSelecting;

		/// The location of the cursor when the mouse was first clicked
		short m_Start[2];

		/// The location of the cursor when the mouse gets released
		short m_End[2];

		/// The View we are doing the selection in.
		M3dView m_View;

	};

	//-----------------------------------------------------------------------------------	RectangleSelectToolCommand
	/// \brief	This class represents a simple mel command that will allow
	/// 		the user to enter a context for our tool. It simply has to return
	/// 		an instance of the Sontext class.
	///
	class RectangleSelectToolCommand
		: public MPxContextCommand
	{
	public:

		/// \brief	This function needs to return a context for maya to use
		/// 		when our tool requires input from the viewport.
		/// \return	a new instance of RectangleSelectTool
		///
		virtual MPxContext* makeObj();

		/// \brief	This returns a new instance of the tool command for maya to use.
		/// \return	an instance of RectangleSelectContextCmd
		///
		static void*		creator();
	};


#endif
